import { NextFunction, Request, Response, Router } from "express";
import { BaseRoute } from "./route";
import * as redis from "redis";
import { RedisClient } from "redis";



/**
 * / route
 *
 * @class User
 */
export class PravilaRoute extends BaseRoute {

  public static client: RedisClient = redis.createClient();

  /**
   * Create the routes.
   *
   * @class PravilaRoute
   * @method create
   * @static
   */
  public static create(router: Router) {
    //log
    console.log("[PravilaRoute::create] Creating pravila route.");

    //add pravila route
    router.get("/pravila", (req: Request, res: Response, next: NextFunction) => {
      new PravilaRoute().pravila(req, res, next);
    });
    router.post("/pravila/filtri/add", (req: Request, res: Response, next: NextFunction) => {
      new PravilaRoute().pravilaAdd(req, res, next);
    });
    router.get("/pravila/filtri/delete/:id", (req: Request, res: Response, next: NextFunction) => {
      new PravilaRoute().pravilaDelete(req, res, next);
    });

    const client: RedisClient = redis.createClient()
  }

  /**
   * Constructor
   *
   * @class PravilaRoute
   * @constructor
   */
  constructor() {
    super();
  }

  /**
   * The pravila route.
   *
   * @class PravilaRoute
   * @method pravila
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public pravila(req: Request, res: Response, next: NextFunction) {
    //set custom title
    this.title = "Pravila";

    PravilaRoute.client.lrange('filtri', 0, -1, (err, result) => {
      if(err) {
        console.log(err)
      } else {
        //set options
        let options: Object = {
          "message": "Welcome to the Tour of Heros! Pravila"
        };

        //render template
        this.render(req, res, "pravila", {seznamFiltrov: result});
      }
    })
  }

  /**
   * The pravila/filtri/delete route.
   *
   * @class PravilaRoute
   * @method pravilaAdd
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public pravilaDelete(req: Request, res: Response, next: NextFunction) {
    //set custom title
    this.title = "Pravila";

    PravilaRoute.client.lset('filtri', req.params.id, '__deleted__', (err, result) => {
      console.log(req.baseUrl);
      if(err) {
        console.log(err)
      }
    })
    PravilaRoute.client.lrem('filtri', 0, '__deleted__', (err, result) => {
      if(err) {
        console.log(err)
      }
    })
    res.redirect('/pravila');
  }

  /**
   * The pravila/filtri/add route.
   *
   * @class PravilaRoute
   * @method pravilaAdd
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public pravilaAdd(req: Request, res: Response, next: NextFunction) {
    //set custom title
    this.title = "Pravila";

    console.log(req.baseUrl);
    PravilaRoute.client.lpush('filtri', req.body.filtri, (err, result) => {
      if(err) {
        console.log(err)
      } else {
        //render template
        res.redirect('/pravila');
        //this.render(req, res, "pravila", {seznamFiltrov: result});
      }
    })
  }
}